@@ -19,5 +19,5 @@
 
 
 def includeme(config):
-    config.add_route('graphql', '/graphql')
+    config.add_route("graphql", "/graphql")
     config.scan(__name__)
