@@ -1,5 +1,5 @@
-# copyright 2003-2018 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
-# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
+# copyright 2003-2021 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# contact https://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This file is part of CubicWeb.
 #
@@ -14,7 +14,7 @@
 # details.
 #
 # You should have received a copy of the GNU Lesser General Public License along
-# with CubicWeb.  If not, see <http://www.gnu.org/licenses/>.
+# with CubicWeb.  If not, see <https://www.gnu.org/licenses/>.
 """Implement the interface of a GraphQL schema for YAMS"""
 
 import binascii
@@ -26,23 +26,42 @@ import json
 
 from cubicweb_graphql import utils
 
-TYPE_FIELD_META = "_meta"  # Name of the meta field for a GraphQL object type class
-_TYPE_FIELD_SCHEMA = "__schema"  # Attribute name for the reference to the YAMS entity schema from a GraphQL object type
-FIELD_SUB_SEPARATOR = "_as_"  # Separator between a field name and its type in generated typed sub-fields (e.g. thing_as_Stuff)
-_FIELD_CREATE = "create"  # Prefix for the mutation fields for object creators
-_FIELD_UPDATE = "update"  # Prefix for the mutation fields for object updaters
-_FIELD_DELETE = "delete"  # Prefix for the mutation fields for object removers
-_FIELD_DELETE_FROM = "deleteFrom"  # Prefix for the mutation fields for relation removers
-ARG_UPDATE_OLD = "old"  # Name of the argument in an updater for the values to be matched
-ARG_UPDATE_NEW = "new"  # Name of the argument in an updated for the new value
-ARG_DELETE_FROM = "from"  # Name of the argument in a delete relation for the targeted entities
-_TYPE_META = "Meta"  # Name for the meta-class of a GraphQL object type
-_TYPE_FIELDS = "Fields"  # Suffix for the graphene types of inheritable data
-_TYPE_INPUT = "InputOf"  # Prefix for the graphene type for input types
-_TYPE_CREATOR = "Create"  # Prefix for the graphene type for creator mutation types
-_TYPE_UNION = "Union"  # Prefix for the graphene anonymous union types
-_TYPE_QUERY = "Query"  # Suffix for the graphene type for the top query type
-_TYPE_MUTATION = "Mutation"  # Suffix for the graphene type for the top mutation type
+# Name of the meta field for a GraphQL object type class
+TYPE_FIELD_META = "_meta"
+# Attribute name for the reference to
+# the YAMS entity schema from a GraphQL object type
+_TYPE_FIELD_SCHEMA = "__schema"
+# Separator between a field name and its type in generated typed sub-fields
+# (e.g. thing_as_Stuff)
+FIELD_SUB_SEPARATOR = "_as_"
+# Prefix for the mutation fields for object creators
+_FIELD_CREATE = "create"
+# Prefix for the mutation fields for object updaters
+_FIELD_UPDATE = "update"
+# Prefix for the mutation fields for object removers
+_FIELD_DELETE = "delete"
+# Prefix for the mutation fields for relation removers
+_FIELD_DELETE_FROM = "deleteFrom"
+# Name of the argument in an updater for the values to be matched
+ARG_UPDATE_OLD = "old"
+# Name of the argument in an updated for the new value
+ARG_UPDATE_NEW = "new"
+# Name of the argument in a delete relation for the targeted entities
+ARG_DELETE_FROM = "from"
+# Name for the meta-class of a GraphQL object type
+_TYPE_META = "Meta"
+# Suffix for the graphene types of inheritable data
+_TYPE_FIELDS = "Fields"
+# Prefix for the graphene type for input types
+_TYPE_INPUT = "InputOf"
+# Prefix for the graphene type for creator mutation types
+_TYPE_CREATOR = "Create"
+# Prefix for the graphene anonymous union types
+_TYPE_UNION = "Union"
+# Suffix for the graphene type for the top query type
+_TYPE_QUERY = "Query"
+# Suffix for the graphene type for the top mutation type
+_TYPE_MUTATION = "Mutation"
 
 
 class ScalarBytes(graphene.Scalar):
@@ -126,7 +145,7 @@ class ScalarJsonb(graphene.Scalar):
 
     @staticmethod
     def serialize(value):
-        if isinstance(value, str) or isinstance(value, unicode):
+        if isinstance(value, str):
             return json.loads(value)
         return value
 
@@ -155,7 +174,7 @@ _PRIMITIVES = {
     "BigInt": ScalarBigInt,
     "Decimal": ScalarDecimal,
     "Interval": ScalarInterval,
-    "Jsonb": ScalarJsonb
+    "Jsonb": ScalarJsonb,
 }
 
 
@@ -180,9 +199,11 @@ class GraphQLSchemaTypes:
         self.unions = {}
         # Map of YAMS entity names to their GraphQL input representation
         self.inputs = {}
-        # Map of YAMS entity names to their corresponding list of possible arguments
+        # Map of YAMS entity names to their corresponding list
+        # of possible arguments
         self.arguments = {}
-        # Map of YAMS entity names to their corresponding GraphQL updater mutation types
+        # Map of YAMS entity names to
+        # their corresponding GraphQL updater mutation types
         self.updaters = {}
         # All known GraphQL types
         self.all_types = []
@@ -208,7 +229,9 @@ class GraphQLSchemaTypes:
         if common is not None and common.type + _TYPE_FIELDS in self.blobs:
             base_classes += (self.blobs[common.type + _TYPE_FIELDS],)
         member_types = tuple([self.entities[member.type] for member in members])
-        union_members = {_TYPE_META: type(_TYPE_META, (object,), {"types": member_types})}
+        union_members = {
+            _TYPE_META: type(_TYPE_META, (object,), {"types": member_types})
+        }
         result = type(_TYPE_UNION + str(len(self.unions)), base_classes, union_members)
         self.unions[key] = result
         self.all_types.append(result)
@@ -238,7 +261,8 @@ class GraphQLSchemaTypes:
     @staticmethod
     def _get_ancestry_of(entity):
         """
-        Get the ancestry of this entity, beginning from the most abstract and terminating by this entity
+        Get the ancestry of this entity,
+        beginning from the most abstract and terminating by this entity
         :param entity: The entity
         :return: The ancestry
         """
@@ -264,73 +288,167 @@ class GraphQLSchemaTypes:
         :param entity_schema: The schema of a YAMS entity
         :return: Nothing
         """
-        fields = {  # Data fields
-            _TYPE_FIELD_SCHEMA: entity_schema
-        }
+        fields = {_TYPE_FIELD_SCHEMA: entity_schema}  # Data fields
         for relation in entity_schema.subject_relations():
             if relation.type == "eid":
                 # eid is treated as the identifier
+                def resolver_lambda(
+                    instance,
+                    info,
+                    relation_name=relation.type,
+                    resolver=self.resolver,
+                    *args,
+                    **kwargs
+                ):
+                    return resolver.resolve_primitive_field(
+                        instance, relation_name, graphene.ID, info, *args, **kwargs
+                    )
+
                 fields["eid"] = graphene.Field(
                     graphene.ID,
                     name="eid",
-                    resolver=lambda instance, info, relation_name=relation.type, resolver=self.resolver, *args, **kwargs:
-                    resolver.resolve_primitive_field(instance, relation_name, graphene.ID, info, *args, **kwargs))
+                    resolver=resolver_lambda,
+                )
             elif relation.final:
                 # handle a primitive type
                 field_type = relation.objects(entity_schema)[0].type
                 if field_type in _PRIMITIVES:
                     target_type = _PRIMITIVES[field_type]
+
+                    def resolver_lambda(
+                        instance,
+                        info,
+                        relation_name=relation.type,
+                        target_type=target_type,
+                        resolver=self.resolver,
+                        *args,
+                        **kwargs
+                    ):
+                        return resolver.resolve_primitive_field(
+                            instance, relation_name, target_type, info, *args, **kwargs
+                        )
+
                     fields[relation.type] = graphene.Field(
                         target_type,
                         name=relation.type,
-                        resolver=lambda instance, info, relation_name=relation.type, target_type=target_type, resolver=self.resolver, *args, **kwargs:
-                        resolver.resolve_primitive_field(instance, relation_name, target_type, info, *args, **kwargs))
+                        resolver=resolver_lambda,
+                    )
             else:
                 # handle a reference to another entity
                 objects = relation.objects(entity_schema)
-                definition = entity_schema.rdef(relation, role="subject", targettype=objects[0])
-                is_multi = definition.cardinality[1] == '+' or definition.cardinality[1] == '*'
+                definition = entity_schema.rdef(
+                    relation, role="subject", targettype=objects[0]
+                )
+                is_multi = (
+                    definition.cardinality[1] == "+" or definition.cardinality[1] == "*"
+                )
                 if len(objects) > 1:
-                    target_type = lambda objects=objects, data=self: data._get_union(objects)
+                    target_type = lambda objects=objects, data=self: data._get_union(
+                        objects
+                    )
                     arguments = self._get_arguments_union(objects)
                 else:
-                    target_type = lambda entities=self.entities, n=objects[0].type: entities[n]
+                    target_type = lambda entities=self.entities, n=objects[
+                        0
+                    ].type: entities[n]
                     arguments = self.arguments[objects[0].type]
                 if is_multi:
+
+                    def resolver_lambda(
+                        instance,
+                        info,
+                        relation_name=relation.type,
+                        target_type=target_type,
+                        resolver=self.resolver,
+                        *args,
+                        **kwargs
+                    ):
+                        return resolver.resolve_vector_object_field(
+                            instance,
+                            relation_name,
+                            target_type(),
+                            info,
+                            *args,
+                            **kwargs
+                        )
+
                     fields[relation.type] = graphene.Field(
                         graphene.List(graphene.NonNull(target_type)),
                         name=relation.type,
                         args=arguments,
-                        resolver=lambda instance, info, relation_name=relation.type, target_type=target_type, resolver=self.resolver, *args, **kwargs:
-                        resolver.resolve_vector_object_field(instance, relation_name, target_type(), info, *args, **kwargs))
+                        resolver=resolver_lambda,
+                    )
                 else:
+
+                    def resolver_lambda(
+                        instance,
+                        info,
+                        relation_name=relation.type,
+                        target_type=target_type,
+                        resolver=self.resolver,
+                        *args,
+                        **kwargs
+                    ):
+                        return resolver.resolve_scalar_object_field(
+                            instance,
+                            relation_name,
+                            target_type(),
+                            info,
+                            *args,
+                            **kwargs
+                        )
+
                     fields[relation.type] = graphene.Field(
                         target_type,
                         name=relation.type,
                         args=arguments,
-                        resolver=lambda instance, info, relation_name=relation.type, target_type=target_type, resolver=self.resolver, *args, **kwargs:
-                        resolver.resolve_scalar_object_field(instance, relation_name, target_type(), info, *args, **kwargs))
+                        resolver=resolver_lambda,
+                    )
 
         super_entity = entity_schema.specializes()
         sub_entities = entity_schema.specialized_by()
         if len(sub_entities) > 0:
-            # Produce a EntityFields graphene AbstractType and a graph ObjectType that inherits from it
-            fields[TYPE_FIELD_META] = type(_TYPE_META, (object,), {"name": entity_schema.type + _TYPE_FIELDS})
+            # Produce a EntityFields graphene AbstractType
+            # and a graph ObjectType that inherits from it
+            fields[TYPE_FIELD_META] = type(
+                _TYPE_META, (object,), {"name": entity_schema.type + _TYPE_FIELDS}
+            )
             if super_entity is not None:
                 # Intermediate entity (is a specialization and can be specialized)
-                super_type_fields = self.blobs[super_entity.type + _TYPE_FIELDS] if super_entity is not None else None
-                class_fields = type(entity_schema.type + _TYPE_FIELDS, (super_type_fields,), fields)
+                super_type_fields = (
+                    self.blobs[super_entity.type + _TYPE_FIELDS]
+                    if super_entity is not None
+                    else None
+                )
+                class_fields = type(
+                    entity_schema.type + _TYPE_FIELDS, (super_type_fields,), fields
+                )
             else:
                 # Can be specialized, but is not a specialization
-                class_fields = type(entity_schema.type + _TYPE_FIELDS, (object,), fields)
-            class_entity = type(entity_schema.type, (graphene.ObjectType, class_fields), {})
+                class_fields = type(
+                    entity_schema.type + _TYPE_FIELDS, (object,), fields
+                )
+            class_entity = type(
+                entity_schema.type, (graphene.ObjectType, class_fields), {}
+            )
             self.blobs[class_fields.__name__] = class_fields
             self.entities[class_entity.__name__] = class_entity
             self.all_types.append(class_entity)
         elif super_entity is not None:
             # This is a specialized entity
-            super_type_fields = self.blobs[super_entity.type + _TYPE_FIELDS] if super_entity is not None else None
-            class_entity = type(entity_schema.type, (graphene.ObjectType, super_type_fields,), fields)
+            super_type_fields = (
+                self.blobs[super_entity.type + _TYPE_FIELDS]
+                if super_entity is not None
+                else None
+            )
+            class_entity = type(
+                entity_schema.type,
+                (
+                    graphene.ObjectType,
+                    super_type_fields,
+                ),
+                fields,
+            )
             self.entities[class_entity.__name__] = class_entity
             self.all_types.append(class_entity)
         else:
@@ -345,39 +463,45 @@ class GraphQLSchemaTypes:
         :param entity_schema: The schema of a YAMS entity
         :return: Nothing
         """
-        fields = {  # Data fields
-            _TYPE_FIELD_SCHEMA: entity_schema
-        }
+        fields = {_TYPE_FIELD_SCHEMA: entity_schema}  # Data fields
         for relation in entity_schema.subject_relations():
             if relation.type == "eid":
                 # eid is treated as the identifier
-                fields["eid"] = graphene.Field(
-                    graphene.ID,
-                    name="eid")
+                fields["eid"] = graphene.Field(graphene.ID, name="eid")
             elif relation.final:
                 # handle a primitive type
                 field_type = relation.objects(entity_schema)[0].type
                 if field_type in _PRIMITIVES:
                     target_type = _PRIMITIVES[field_type]
                     fields[relation.type] = graphene.Field(
-                        target_type,
-                        name=relation.type)
+                        target_type, name=relation.type
+                    )
             else:
                 # handle a reference to another entity
                 objects = relation.objects(entity_schema)
                 if len(objects) > 1:
                     for target_schema in objects:
-                        target_type = lambda inputs=self.inputs, n=_TYPE_INPUT + target_schema.type: inputs[n]
+
+                        def target_type(
+                            inputs=self.inputs, n=_TYPE_INPUT + target_schema.type
+                        ):
+                            return inputs[n]
+
                         name = relation.type + FIELD_SUB_SEPARATOR + target_schema.type
-                        fields[name] = graphene.Field(
-                            target_type,
-                            name=name)
+                        fields[name] = graphene.Field(target_type, name=name)
                 else:
-                    target_type = lambda inputs=self.inputs, n=_TYPE_INPUT + objects[0].type: inputs[n]
+
+                    def target_type(
+                        inputs=self.inputs, n=_TYPE_INPUT + objects[0].type
+                    ):
+                        return inputs[n]
+
                     fields[relation.type] = graphene.Field(
-                        target_type,
-                        name=relation.type)
-        class_entity = type(_TYPE_INPUT + entity_schema.type, (graphene.InputObjectType,), fields)
+                        target_type, name=relation.type
+                    )
+        class_entity = type(
+            _TYPE_INPUT + entity_schema.type, (graphene.InputObjectType,), fields
+        )
         self.inputs[class_entity.__name__] = class_entity
         self.all_types.append(class_entity)
 
@@ -392,35 +516,41 @@ class GraphQLSchemaTypes:
             if relation.type == "eid":
                 # eid is treated as the identifier
                 arguments["eid"] = graphene.Argument(
-                    graphene.ID,
-                    name="eid",
-                    required=False)
+                    graphene.ID, name="eid", required=False
+                )
             elif relation.final:
                 # handle a primitive type
                 field_type = relation.objects(entity_schema)[0].type
                 if field_type in _PRIMITIVES:
                     target_type = _PRIMITIVES[field_type]
                     arguments[relation.type] = graphene.Argument(
-                        target_type,
-                        name=relation.type,
-                        required=False)
+                        target_type, name=relation.type, required=False
+                    )
             else:
                 # handle a reference to another entity
                 objects = relation.objects(entity_schema)
                 if len(objects) > 1:
                     for target_schema in objects:
                         name = relation.type + FIELD_SUB_SEPARATOR + target_schema.type
-                        target_type = lambda inputs=self.inputs, n=_TYPE_INPUT + target_schema.type: inputs[n]
+
+                        def target_type(
+                            inputs=self.inputs, n=_TYPE_INPUT + target_schema.type
+                        ):
+                            return inputs[n]
+
                         arguments[name] = graphene.Argument(
-                            target_type,
-                            name=name,
-                            required=False)
+                            target_type, name=name, required=False
+                        )
                 else:
-                    target_type = lambda inputs=self.inputs, n=_TYPE_INPUT + objects[0].type: inputs[n]
+
+                    def target_type(
+                        inputs=self.inputs, n=_TYPE_INPUT + objects[0].type
+                    ):
+                        return inputs[n]
+
                     arguments[relation.type] = graphene.Argument(
-                        target_type,
-                        name=relation.type,
-                        required=False)
+                        target_type, name=relation.type, required=False
+                    )
         self.arguments[entity_schema.type] = arguments
 
     def _create_entity_query_field(self, entity_schema):
@@ -430,12 +560,23 @@ class GraphQLSchemaTypes:
         :return: The corresponding field
         """
         entity_class = self.entities[entity_schema.type]
+
+        def resolver_lambda(
+            self,
+            info,
+            entity_type=entity_class,
+            resolver=self.resolver,
+            *args,
+            **kwargs
+        ):
+            return resolver.resolve_entities(entity_type, self, info, *args, **kwargs)
+
         return graphene.Field(
             graphene.List(graphene.NonNull(entity_class)),
             name=entity_schema.type,
             args=self.arguments[entity_schema.type],
-            resolver=lambda self, info, entity_type=entity_class, resolver=self.resolver, *args, **kwargs:
-            resolver.resolve_entities(entity_type, self, info, *args, **kwargs))
+            resolver=resolver_lambda,
+        )
 
     def _create_query_type(self):
         """
@@ -447,7 +588,9 @@ class GraphQLSchemaTypes:
             if entity.final:
                 continue
             members[entity.type] = self._create_entity_query_field(entity)
-        self.query_type = type(self.schema.name + _TYPE_QUERY, (graphene.ObjectType,), members)
+        self.query_type = type(
+            self.schema.name + _TYPE_QUERY, (graphene.ObjectType,), members
+        )
 
     def _create_entity_creator_field(self, entity_schema):
         """
@@ -456,12 +599,23 @@ class GraphQLSchemaTypes:
         :return: The corresponding field
         """
         entity_class = self.entities[entity_schema.type]
+
+        def resolver_lambda(
+            self,
+            info,
+            entity_type=entity_class,
+            resolver=self.resolver,
+            *args,
+            **kwargs
+        ):
+            return resolver.on_create_entity(self, info, entity_type, **kwargs)
+
         return graphene.Field(
             graphene.NonNull(entity_class),
             name=_FIELD_CREATE + entity_schema.type,
             args=self.arguments[entity_schema.type],
-            resolver=lambda self, info, entity_type=entity_class, resolver=self.resolver, *args, **kwargs:
-            resolver.on_create_entity(self, info, entity_type, **kwargs))
+            resolver=resolver_lambda,
+        )
 
     def _create_entity_updater_field(self, entity_schema):
         """
@@ -474,20 +628,31 @@ class GraphQLSchemaTypes:
             ARG_UPDATE_OLD: graphene.Argument(
                 self.inputs[_TYPE_INPUT + entity_schema.type],
                 name=ARG_UPDATE_OLD,
-                required=False
+                required=False,
             ),
             ARG_UPDATE_NEW: graphene.Argument(
                 self.inputs[_TYPE_INPUT + entity_schema.type],
                 name=ARG_UPDATE_NEW,
-                required=False
-            )
+                required=False,
+            ),
         }
+
+        def resolver_lambda(
+            self,
+            info,
+            entity_type=entity_class,
+            resolver=self.resolver,
+            *args,
+            **kwargs
+        ):
+            return resolver.on_update_entity(self, info, entity_type, **kwargs)
+
         return graphene.Field(
             graphene.List(graphene.NonNull(entity_class)),
             name=_FIELD_UPDATE + entity_schema.type,
             args=arguments,
-            resolver=lambda self, info, entity_type=entity_class, resolver=self.resolver, *args, **kwargs:
-            resolver.on_update_entity(self, info, entity_type, **kwargs))
+            resolver=resolver_lambda,
+        )
 
     def _create_entity_deleter_field(self, entity_schema):
         """
@@ -496,12 +661,23 @@ class GraphQLSchemaTypes:
         :return: The corresponding field
         """
         entity_class = self.entities[entity_schema.type]
+
+        def resolver_lambda(
+            self,
+            info,
+            entity_type=entity_class,
+            resolver=self.resolver,
+            *args,
+            **kwargs
+        ):
+            return resolver.on_delete_entity(self, info, entity_type, **kwargs)
+
         return graphene.Field(
             graphene.List(graphene.ID),
             name=_FIELD_DELETE + entity_schema.type,
             args=self.arguments[entity_schema.type],
-            resolver=lambda self, info, entity_type=entity_class, resolver=self.resolver, *args, **kwargs:
-            resolver.on_delete_entity(self, info, entity_type, **kwargs))
+            resolver=resolver_lambda,
+        )
 
     def _create_entity_relation_deleter_field(self, entity_schema):
         """
@@ -514,7 +690,7 @@ class GraphQLSchemaTypes:
             ARG_DELETE_FROM: graphene.Argument(
                 self.inputs[_TYPE_INPUT + entity_schema.type],
                 name=ARG_DELETE_FROM,
-                required=False
+                required=False,
             ),
         }
         for name, arg in self.arguments[entity_schema.type].items():
@@ -523,12 +699,23 @@ class GraphQLSchemaTypes:
                 # primitive field
                 continue
             arguments[name] = arg
+
+        def resolver_lambda(
+            self,
+            info,
+            entity_type=entity_class,
+            resolver=self.resolver,
+            *args,
+            **kwargs
+        ):
+            return resolver.on_delete_relation(self, info, entity_type, **kwargs)
+
         return graphene.Field(
             graphene.List(graphene.ID),
             name=_FIELD_DELETE_FROM + entity_schema.type,
             args=arguments,
-            resolver=lambda self, info, entity_type=entity_class, resolver=self.resolver, *args, **kwargs:
-            resolver.on_delete_relation(self, info, entity_type, **kwargs))
+            resolver=resolver_lambda,
+        )
 
     def _create_mutation_type(self):
         """
@@ -539,11 +726,21 @@ class GraphQLSchemaTypes:
         for entity in self.schema.entities():
             if entity.final:
                 continue
-            members[_FIELD_CREATE + entity.type] = self._create_entity_creator_field(entity)
-            members[_FIELD_UPDATE + entity.type] = self._create_entity_updater_field(entity)
-            members[_FIELD_DELETE + entity.type] = self._create_entity_deleter_field(entity)
-            members[_FIELD_DELETE_FROM + entity.type] = self._create_entity_relation_deleter_field(entity)
-        self.mutation_type = type(self.schema.name + _TYPE_MUTATION, (graphene.ObjectType,), members)
+            members[_FIELD_CREATE + entity.type] = self._create_entity_creator_field(
+                entity
+            )
+            members[_FIELD_UPDATE + entity.type] = self._create_entity_updater_field(
+                entity
+            )
+            members[_FIELD_DELETE + entity.type] = self._create_entity_deleter_field(
+                entity
+            )
+            members[
+                _FIELD_DELETE_FROM + entity.type
+            ] = self._create_entity_relation_deleter_field(entity)
+        self.mutation_type = type(
+            self.schema.name + _TYPE_MUTATION, (graphene.ObjectType,), members
+        )
 
     def _produce(self):
         """
@@ -570,7 +767,12 @@ class GraphQLSchemaTypes:
                     remainings.append(entity)
         self._create_query_type()
         self._create_mutation_type()
-        self.graphql_schema = graphene.Schema(query=self.query_type, mutation=self.mutation_type, auto_camelcase=False, types=self.all_types)
+        self.graphql_schema = graphene.Schema(
+            query=self.query_type,
+            mutation=self.mutation_type,
+            auto_camelcase=False,
+            types=self.all_types,
+        )
 
 
 def create_graphql_schema(schema, resolver):
