@@ -27,8 +27,8 @@ class GraphQLStartupHook(hook.Hook):
     Hook for the startup of the GraphQL querier
     """
 
-    __regid__ = 'cw.graphql.startup'
-    events = ('server_startup',)
+    __regid__ = "cw.graphql.startup"
+    events = ("server_startup",)
 
     def __call__(self):
         GraphQLQuerier.register(self.repo)
