@@ -74,7 +74,9 @@ class _GraphQLQuerierPart(object):
         :param kwargs: The mapping of values for the named variables in the query
         :return: The GraphQL execution result
         """
-        return self._query_schema.execute(query, context_value={CONNECTION: connection}, variable_values=kwargs)
+        return self._query_schema.execute(
+            query, context_value={CONNECTION: connection}, variable_values=kwargs
+        )
 
 
 def default_format_error(error):
@@ -85,7 +87,7 @@ def default_format_error(error):
     """
     if isinstance(error, GraphQLError):
         return format_graphql_error(error)
-    return {'message': six.text_type(error)}
+    return {"message": six.text_type(error)}
 
 
 def format_execution_result(execution_result, format_error=default_format_error):
@@ -98,9 +100,7 @@ def format_execution_result(execution_result, format_error=default_format_error)
     if execution_result:
         response = {}
         if execution_result.errors:
-            response['errors'] = [
-                format_error(e) for e in execution_result.errors
-            ]
+            response["errors"] = [format_error(e) for e in execution_result.errors]
         if not execution_result.invalid:
-            response['data'] = execution_result.data
+            response["data"] = execution_result.data
         return response
