@@ -16,18 +16,25 @@
 # You should have received a copy of the GNU Lesser General Public License along
 # with CubicWeb.  If not, see <http://www.gnu.org/licenses/>.
 """Implement the resolver for CubicWeb entities"""
-from cubicweb_graphql.gql_schema import FIELD_SUB_SEPARATOR, TYPE_FIELD_META, ARG_UPDATE_OLD, ARG_UPDATE_NEW, ARG_DELETE_FROM
+from cubicweb_graphql.gql_schema import (
+    FIELD_SUB_SEPARATOR,
+    TYPE_FIELD_META,
+    ARG_UPDATE_OLD,
+    ARG_UPDATE_NEW,
+    ARG_DELETE_FROM,
+)
 
 CONNECTION = "__connection"  # The key for the current connection in the context of a GraphQL query
 _CACHE = "__cache"  # The key for the object cache in the context of a GraphQL query
-_MODEL = "__model"  # The attribute name that refers to the CubicWeb entity from the GraphQL object instance
+# The attribute name that refers to the CubicWeb entity from the GraphQL object instance
+_MODEL = "__model"
 _SUBJECT_ARG = "_subject"  # argument name for the current subject of a query
 _ARG_PREFIX = "arg"  # Prefix for generated query arguments
 _SUBJECT_VAR = "X"  # Name of the first subject in generated RQL queries
 
 
 class Resolver:
-    """The resolver is in charge of fetching the backing data for the GraphQL query execution engine"""
+    "The resolver is in charge of fetching the backing data for the GraphQL query execution engine"
 
     def __init__(self, repository, yams_schema, graphql_schema):
         """
@@ -82,7 +89,9 @@ class Resolver:
         try:
             entity = connection.entity_cache(eid)
         except KeyError:
-            entity = connection.vreg['etypes'].etype_class(entity_type.__name__)(connection)
+            entity = connection.vreg["etypes"].etype_class(entity_type.__name__)(
+                connection
+            )
             entity.eid = eid
             connection.set_entity_cache(entity)
         return Resolver._get_graphql_entity(cache, entity_type, entity)
@@ -130,7 +139,9 @@ class Resolver:
         return constraints, new_arguments
 
     @staticmethod
-    def _build_constraints(subject, restrictions, constraints, new_arguments, sub_start=0):
+    def _build_constraints(
+        subject, restrictions, constraints, new_arguments, sub_start=0
+    ):
         """
         Build the set of constraints applicable for a RQL query
         :param subject: The subject of the constraints
@@ -143,14 +154,20 @@ class Resolver:
         sub = sub_start
         for field, value in restrictions.items():
             if FIELD_SUB_SEPARATOR in field:
-                field = field[0:field.index(FIELD_SUB_SEPARATOR)]
+                field = field[0 : field.index(FIELD_SUB_SEPARATOR)]
             if hasattr(value, TYPE_FIELD_META):
                 # this is a complex value
-                sub_restrictions = {k: v for k, v in value.__dict__.items() if not k.startswith("_") and v is not None}
-                sub_subject = subject + str(chr(ord('A') + sub))
+                sub_restrictions = {
+                    k: v
+                    for k, v in value.__dict__.items()
+                    if not k.startswith("_") and v is not None
+                }
+                sub_subject = subject + str(chr(ord("A") + sub))
                 sub += 1
                 constraints.append(subject + " " + field + " " + sub_subject)
-                Resolver._build_constraints(sub_subject, sub_restrictions, constraints, new_arguments)
+                Resolver._build_constraints(
+                    sub_subject, sub_restrictions, constraints, new_arguments
+                )
             else:
                 # simple value
                 arg_name = _ARG_PREFIX + str(len(new_arguments))
@@ -183,9 +200,14 @@ class Resolver:
                 first = False
                 rql += " " + constraint
         results = connection.execute(rql, arguments)
-        return [self._get_graphql_entity(cache, entity_type, entity) for entity in results.entities()]
+        return [
+            self._get_graphql_entity(cache, entity_type, entity)
+            for entity in results.entities()
+        ]
 
-    def resolve_primitive_field(self, instance, relation_name, target_type, info, *args, **kwargs):
+    def resolve_primitive_field(
+        self, instance, relation_name, target_type, info, *args, **kwargs
+    ):
         """
         Fetch the values for an entity's field when its type is primitive
         :param instance: The entity's instance
@@ -199,7 +221,9 @@ class Resolver:
         model = getattr(instance, _MODEL)
         return getattr(model, relation_name)
 
-    def resolve_scalar_object_field(self, instance, relation_name, target_type, info, *args, **kwargs):
+    def resolve_scalar_object_field(
+        self, instance, relation_name, target_type, info, *args, **kwargs
+    ):
         """
         Fetch the values for an entity's field when its type is a single instance of an entity
         :param instance: The entity's instance
@@ -210,11 +234,16 @@ class Resolver:
         :param kwargs: The query parameters
         :return: The resolved value
         """
-        return self._resolve_object_field(instance, relation_name, target_type, info, True, *args, **kwargs)
+        return self._resolve_object_field(
+            instance, relation_name, target_type, info, True, *args, **kwargs
+        )
 
-    def resolve_vector_object_field(self, instance, relation_name, target_type, info, *args, **kwargs):
+    def resolve_vector_object_field(
+        self, instance, relation_name, target_type, info, *args, **kwargs
+    ):
         """
-        Fetch the values for an entity's field when its type is a collection of instances of other entities
+        Fetch the values for an entity's field when its type is a collection of instances
+        of other entities
         :param instance: The entity's instance
         :param relation_name: The name of the relation for this field
         :param target_type: The GraphQL type for this field
@@ -223,9 +252,13 @@ class Resolver:
         :param kwargs: The query parameters
         :return: The resolved value
         """
-        return self._resolve_object_field(instance, relation_name, target_type, info, False, *args, **kwargs)
+        return self._resolve_object_field(
+            instance, relation_name, target_type, info, False, *args, **kwargs
+        )
 
-    def _resolve_object_field(self, instance, relation_name, target_type, info, scalar, *args, **kwargs):
+    def _resolve_object_field(
+        self, instance, relation_name, target_type, info, scalar, *args, **kwargs
+    ):
         """
         Fetch the values for an entity's field when its type is another entity
         :param instance: The entity's instance
@@ -253,13 +286,25 @@ class Resolver:
         else:
             # Generate a RQL query and execute it
             connection = info.context[CONNECTION]
-            rql = "Any " + _SUBJECT_VAR + " WHERE S " + relation_name + " " + _SUBJECT_VAR + " AND S eid %(" + _SUBJECT_ARG + ")s"
+            rql = (
+                "Any "
+                + _SUBJECT_VAR
+                + " WHERE S "
+                + relation_name
+                + " "
+                + _SUBJECT_VAR
+                + " AND S eid %("
+                + _SUBJECT_ARG
+                + ")s"
+            )
             constraints, arguments = self._get_constraints(_SUBJECT_VAR, kwargs)
             for constraint in constraints:
                 rql += " AND " + constraint
             arguments[_SUBJECT_ARG] = model.eid
             results = connection.execute(rql, arguments)
-            values = [self._resolve_entity(cache, entity) for entity in results.entities()]
+            values = [
+                self._resolve_entity(cache, entity) for entity in results.entities()
+            ]
             if scalar:
                 return values[0] if len(values) > 0 else None
             return values
@@ -278,14 +323,20 @@ class Resolver:
         sub = 0
         for field, value in arguments.items():
             if FIELD_SUB_SEPARATOR in field:
-                field = field[0:field.index(FIELD_SUB_SEPARATOR)]
+                field = field[0 : field.index(FIELD_SUB_SEPARATOR)]
             if hasattr(value, TYPE_FIELD_META):
                 # this is a complex value
-                sub_restrictions = {k: v for k, v in value.__dict__.items() if not k.startswith("_") and v is not None}
-                sub_subject = _SUBJECT_VAR + str(chr(ord('A') + sub))
+                sub_restrictions = {
+                    k: v
+                    for k, v in value.__dict__.items()
+                    if not k.startswith("_") and v is not None
+                }
+                sub_subject = _SUBJECT_VAR + str(chr(ord("A") + sub))
                 sub += 1
                 fields.append(_SUBJECT_VAR + " " + field + " " + sub_subject)
-                Resolver._build_constraints(sub_subject, sub_restrictions, constraints, new_arguments)
+                Resolver._build_constraints(
+                    sub_subject, sub_restrictions, constraints, new_arguments
+                )
             else:
                 # simple value
                 arg_name = _ARG_PREFIX + str(len(new_arguments))
@@ -315,7 +366,10 @@ class Resolver:
         connection = info.context[CONNECTION]
         cache = Resolver._get_cache_for(info)
         results = connection.execute(rql, arguments)
-        values = [Resolver._get_graphql_entity(cache, entity_type, entity) for entity in results.entities()]
+        values = [
+            Resolver._get_graphql_entity(cache, entity_type, entity)
+            for entity in results.entities()
+        ]
         return values[0] if len(values) > 0 else None
 
     @staticmethod
@@ -332,21 +386,37 @@ class Resolver:
         sub = 0
 
         # gather constraints for matching current entities
-        sub_restrictions = {k: v for k, v in arguments[ARG_UPDATE_OLD].__dict__.items() if not k.startswith("_") and v is not None}
-        sub = Resolver._build_constraints(_SUBJECT_VAR, sub_restrictions, constraints, new_arguments, sub)
+        sub_restrictions = {
+            k: v
+            for k, v in arguments[ARG_UPDATE_OLD].__dict__.items()
+            if not k.startswith("_") and v is not None
+        }
+        sub = Resolver._build_constraints(
+            _SUBJECT_VAR, sub_restrictions, constraints, new_arguments, sub
+        )
 
         # gather new fields to set for the matched entities
-        sub_restrictions = {k: v for k, v in arguments[ARG_UPDATE_NEW].__dict__.items() if not k.startswith("_") and v is not None}
+        sub_restrictions = {
+            k: v
+            for k, v in arguments[ARG_UPDATE_NEW].__dict__.items()
+            if not k.startswith("_") and v is not None
+        }
         for field, value in sub_restrictions.items():
             if FIELD_SUB_SEPARATOR in field:
-                field = field[0:field.index(FIELD_SUB_SEPARATOR)]
+                field = field[0 : field.index(FIELD_SUB_SEPARATOR)]
             if hasattr(value, TYPE_FIELD_META):
                 # this is a complex value
-                value_restrictions = {k: v for k, v in value.__dict__.items() if not k.startswith("_") and v is not None}
-                sub_subject = _SUBJECT_VAR + str(chr(ord('A') + sub))
+                value_restrictions = {
+                    k: v
+                    for k, v in value.__dict__.items()
+                    if not k.startswith("_") and v is not None
+                }
+                sub_subject = _SUBJECT_VAR + str(chr(ord("A") + sub))
                 sub += 1
                 fields.append(_SUBJECT_VAR + " " + field + " " + sub_subject)
-                Resolver._build_constraints(sub_subject, value_restrictions, constraints, new_arguments)
+                Resolver._build_constraints(
+                    sub_subject, value_restrictions, constraints, new_arguments
+                )
             else:
                 # simple value
                 arg_name = _ARG_PREFIX + str(len(new_arguments))
@@ -376,7 +446,12 @@ class Resolver:
         cache = Resolver._get_cache_for(info)
         results = connection.execute(rql, arguments)
         # no description is built for SET queries
-        return [self._resolve_entity_from_eid(cache, entity_type, connection, results.rows[i][0]) for i in range(len(results))]
+        return [
+            self._resolve_entity_from_eid(
+                cache, entity_type, connection, results.rows[i][0]
+            )
+            for i in range(len(results))
+        ]
 
     @staticmethod
     def _get_delete_rql(type_name, arguments):
@@ -420,19 +495,31 @@ class Resolver:
         to_delete = []
         sub = 0
         # gather constraints for matching source entities
-        sub_restrictions = {k: v for k, v in arguments[ARG_DELETE_FROM].__dict__.items() if not k.startswith("_") and v is not None}
-        sub = Resolver._build_constraints(_SUBJECT_VAR, sub_restrictions, constraints, new_arguments, sub)
+        sub_restrictions = {
+            k: v
+            for k, v in arguments[ARG_DELETE_FROM].__dict__.items()
+            if not k.startswith("_") and v is not None
+        }
+        sub = Resolver._build_constraints(
+            _SUBJECT_VAR, sub_restrictions, constraints, new_arguments, sub
+        )
 
         for field, value in arguments.items():
             if field == ARG_DELETE_FROM:
                 continue
             if FIELD_SUB_SEPARATOR in field:
-                field = field[0:field.index(FIELD_SUB_SEPARATOR)]
-            value_restrictions = {k: v for k, v in value.__dict__.items() if not k.startswith("_") and v is not None}
-            sub_subject = _SUBJECT_VAR + str(chr(ord('A') + sub))
+                field = field[0 : field.index(FIELD_SUB_SEPARATOR)]
+            value_restrictions = {
+                k: v
+                for k, v in value.__dict__.items()
+                if not k.startswith("_") and v is not None
+            }
+            sub_subject = _SUBJECT_VAR + str(chr(ord("A") + sub))
             sub += 1
             to_delete.append(_SUBJECT_VAR + " " + field + " " + sub_subject)
-            Resolver._build_constraints(sub_subject, value_restrictions, constraints, new_arguments)
+            Resolver._build_constraints(
+                sub_subject, value_restrictions, constraints, new_arguments
+            )
         rql = "DELETE"
         for i in range(len(to_delete)):
             prefix = ", " if i > 0 else " "
@@ -452,7 +539,9 @@ class Resolver:
         :param kwargs: The query parameters
         :return: The EID of the delete relations
         """
-        rql, arguments = Resolver._get_delete_relation_rql(entity_type.__name__, kwargs or {})
+        rql, arguments = Resolver._get_delete_relation_rql(
+            entity_type.__name__, kwargs or {}
+        )
         connection = info.context[CONNECTION]
         connection.execute(rql, arguments)
         return []
