@@ -26,9 +26,9 @@ from pyramid.view import view_config
 from cubicweb_graphql.querier import GraphQLQuerier, format_execution_result
 
 
-@ajaxfunc(output_type='json')
+@ajaxfunc(output_type="json")
 def graphql(self):
-    query = self._cw.form.get('query')
+    query = self._cw.form.get("query")
     cnx = self._cw.cnx
     results = GraphQLQuerier.execute(cnx, query)
     results = format_execution_result(results)
@@ -46,13 +46,13 @@ def authenticated(func):
 
 
 @view_config(
-    route_name='graphql',
-    request_method='GET',
-    renderer='json',
+    route_name="graphql",
+    request_method="GET",
+    renderer="json",
 )
 @authenticated
 def graphql_pview(request):
-    query = request.params.get('query', '')
+    query = request.params.get("query", "")
     results = GraphQLQuerier.execute(request.cw_cnx, query)
     results = format_execution_result(results)
     return results
